<?php
class EmailCategoria
{
	public $id;
	public $categoria;
	public $tag;
}

class Email
{
	public $id;
	public $nome;	
	public $email;
	public $data_inclusao;
}

class EmailCategoriaDAO implements DAOInterface
{
	private $_crud;

	function __construct($_crud = null)
	{
		$this->_crud = $_crud;

	}

	private function GetDefaultSelectSQL()
	{
		$sql = "SELECT
		EC.id,
		EC.categoria,
		EC.tag
		FROM emails_categorias EC ";

		return $sql;
	}

	private function Select($selectSQL)
	{
		if($this->_crud)
		{
			$result = $this->_crud->GetRecords($selectSQL);

			if($result->success)
			{
				$values = $result->value;
				$objects = array();
				foreach($values as $dados)
				{
					unset($obj);
					$obj = new EmailCategoria;
					$obj->id = $dados["id"];
					$obj->categoria = $dados["categoria"];
					$obj->tag = $dados["tag"];
					$objects[] = $obj;
				}

				return new ReturnStruct(true, $objects, $result->message);
			}
			else
			{
				return new ReturnStruct(false, null, $result->message);
			}
		}
		else
		{
			throw new Exception("CRUD no definido!");
		}
	}

	public function Find($where = "", $orderBy = "EC.categoria", $limit = "")
	{
		$selectSQL = $this->GetDefaultSelectSQL();
		if($where!="")
		{
			$selectSQL .= " WHERE ".$where;
		}

		$selectSQL .= " ORDER BY ".$orderBy;

		if($limit!="")
		{
			$selectSQL .= " LIMIT ".$limit;
		}

		return $this->Select($selectSQL);
	}

	public function FindByKey($fields)
	{
		$selectSQL = $this->GetDefaultSelectSQL();

		if(count($fields) > 0)
		{
			$selectSQL .= " WHERE ";
			foreach($fields as $field)
			{
				$selectSQL .= " (".$field["name"]." = '".$field["value"]."') AND";
			}
		}

		$selectSQL = substr($selectSQL, 0, strlen($selectSQL)-4);

		//echo $selectSQL;

		$result = $this->Select($selectSQL);

		if($result->success)
		{
			$value = $result->value;

			if(count($value) > 0)
			{
				return $value[0];
			}
			else
			{
				throw new Exception("Categoria no encontrada!");
			}
		}
		else
		{
			throw new Exception($result->message);
		}
	}

	public function CountNumRows($where = "")
	{
		$selectSQL = "SELECT COUNT(*) FROM emails_categorias EC";

		if($where!="")
		{
			$selectSQL .= " WHERE ".$where;
		}

		//echo $selectSQL;

		$res = $this->_crud->ExecuteScalar($selectSQL);
		if($res->success)
		{
			return $res->value;
		}
		else
		{
			throw new Exception('Erro ao selecionar total de registros! Erro: '.$res->message);
		}
	}

	public function Insert($object)
	{
		$sql = "INSERT INTO emails_categorias ".
				"(categoria, tag) ".
				"values ( ".
				"'".$object->categoria."', ".
				"'".$object->tag."')";

		$result = $this->_crud->ExecutaSQL($sql, true);

		if($result->success)
		{
			$object->id = $result->message->GetMessage();
			return new ReturnStruct(true, $object, "Registro inserido com sucesso!");
		}
		else
		{
			return $result;
		}

	}

	public function Update($object)
	{
		$sql = "UPDATE emails_categorias SET ".
				"categoria = '".$object->categoria."', ".
				"tag = '".$object->tag."' ".
				"WHERE ".
				"id = ".$object->id;

		$result = $this->_crud->ExecutaSQL($sql);

		if($result->success)
		{
			return new ReturnStruct(true, $object, "Registro alterado com sucesso!");
		}
		else
		{
			return $result;
		}

	}

	public function Delete($object)
	{
		$sql = "DELETE FROM emails_categorias WHERE id = ".$object->id;

		$result = $this->_crud->ExecutaSQL($sql);

		if($result->success)
		{
			return new ReturnStruct(true, null, "Registro removido com sucesso!");
		}
		else
		{
			return $result;
		}

	}

}

class EmailDAO implements DAOInterface
{
	private $_crud;
	function __construct($_crud = null)
	{
		$this->_crud = $_crud;
	}
	private function GetDefaultSelectSQL($full = false)
	{
		$sql = "SELECT ".
				"E.id, ".
				"E.nome, ".				
				"E.data_inclusao, ".
				"E.email ".				
				"FROM emails E ";
		return $sql;
	}
	private function Select($selectSQL)
	{
		//echo $selectSQL;
		if($this->_crud)
		{
			$result = $this->_crud->GetRecords($selectSQL);
			if($result->success)
			{
				$values = $result->value;
				$objects = array();
				foreach($values as $dados)
				{
					unset($obj);
					$obj = new Email();
					$obj->id                = $dados["id"];
					$obj->nome            	= $dados["nome"];
					$obj->email      		= $dados["email"];
					$obj->data_inclusao     = $dados["data_inclusao"];
					$obj->_crud 			= $this->_crud;
					$objects[] = $obj;
				}
				return new ReturnStruct(true, $objects, $result->message);
			}
			else
			{
				return new ReturnStruct(false, null, $result->message);
			}
		}
		else
		{
			throw new Exception("CRUD no definido!");
		}
	}
	public function Find($where = "", $orderBy = "C.id", $qtde="", $full = false)
	{
		$selectSQL = $this->GetDefaultSelectSQL($full);
		if($where!="")
		{
			$selectSQL .= " WHERE ".$where;
		}
		$selectSQL .= " ORDER BY ".$orderBy;
		if($qtde!="")
		{
			$selectSQL .= " LIMIT ".$qtde;
		}
		return $this->Select($selectSQL);
	}
	public function FindByKey($fields, $full = false)
	{
		$selectSQL = $this->GetDefaultSelectSQL($full);
		if(count($fields) > 0)
		{
			$selectSQL .= " WHERE ";
			foreach($fields as $field)
			{
				$selectSQL .= " ".$field["name"]." = '".$field["value"]."' AND";
			}
		}
		$selectSQL = substr($selectSQL, 0, strlen($selectSQL)-4);
		//echo $selectSQL;
		$result = $this->Select($selectSQL);
		if($result->success)
		{
			$value = $result->value;
			if(count($value) > 0)
			{
				return $value[0];
			}
			else
			{
				throw new Exception("Nenhum registro encontrado!");
			}
		}
		else
		{
			throw new Exception($result->message);
		}
	}
	public function CountNumRows($where = "")
	{
		$selectSQL = "SELECT COUNT(*) FROM emails E";
		if($where!="")
		{
			$selectSQL .= " where ".$where;
		}
		//echo $selectSQL;
		$res = $this->_crud->ExecuteScalar($selectSQL);
		if($res->success)
		{
			return $res->value;
		}
		else
		{
			throw new Exception('Erro ao selecionar total de registros! Erro: '.$res->message);
		}
	}
	public function Insert($object)
	{
		$sql = "INSERT INTO emails ".
		"(nome, email, data_inclusao) ".
		"values ( ".
		"'".$object->nome."', ".
		"'".$object->email."', ".
		"'".$object->data_inclusao."')";
		$result = $this->_crud->ExecutaSQL($sql, true);
		if($result->success)
		{
			$object->id = $result->message->GetMessage();
			return new ReturnStruct(true, $object, "Registro inserido com sucesso!");
		}
		else
		{
			return $result;
		}
	}
	public function Update($object)
	{
		$sql = "UPDATE emails SET ".
		"nome = '".$object->nome."', ".
		"email = '".$object->email."' ".		
		" WHERE ".
		"id = ".$object->id;
		$result = $this->_crud->ExecutaSQL($sql);
		if($result->success)
		{
			return new ReturnStruct(true, $object, "Registro alterado com sucesso!");
		}
		else
		{
			return $result;
		}
	}
	public function Delete($object)
	{
		$sql = "DELETE FROM emails WHERE id = ".$object->id;
		$result = $this->_crud->ExecutaSQL($sql);
		if($result->success)
		{
			return new ReturnStruct(true, null, "Registro removido com sucesso!");
		}
		else
		{
			return $result;
		}
	}
	
	public function AddEmailToCategoria($email, $tag)
	{
		
		try 
		{
			$emailObj = $this->FindByKey(array(array("name"=>"E.email", "value"=>$email)));
		}
		catch(Exception $ex)
		{
			
		}
		
		if(!$emailObj)
		{
			$object = new Email();
			$object->email = $email;
			$object->data_inclusao = date("Y-m-d H:i:s");
			$res = $this->Insert($object);
			$emailObj = $res->value;
		}

			
		$crudCat = new EmailCategoriaDAO($this->_crud);
		try 
		{
			$cat = $crudCat->FindByKey(array(array("name"=>"EC.tag", "value"=>$tag)));
			
			$sql = "INSERT INTO emails_categorias_rel (id_email, id_categoria) VALUES ('".$emailObj->id."', '".$cat->id."')";
			$result = $this->_crud->ExecutaSQL($sql);
			if($result->success)
			{
				return new ReturnStruct(true, null, "E-mail adicionado a categoria com sucesso!");
			}
			else
			{
				return $result;
			}
			
		}
		catch(Exception $ex)
		{
			return new ReturnStruct(false, null, "Categoria invlida!");
		}
		
	}
}
?>